 /*
 * file      : src/draw.c
 * project   : tm
 * with      : Gtk-2
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TM - Edition des heures de travail / Edition of working hours
 * GNU General Public License
 * 
 * This file is part of TM.
 * 
 * TM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 * 
 * TM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TM.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
	#define gettext_noop(x) (x) 
#endif

#include <gtk/gtk.h>
#include <cairo.h>
#include <cairo-pdf.h>
#include <cairo-ps.h>
#include <cairo-xlib.h>
#include <X11/Xlib.h>
#include <string.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_tm/lib.h"
#endif

#include "global.h"
#include "callbacks.h"



// 
// FORCE LE REDRAW DE LA SURFACE DE DESSIN
// 
void draw_year( void )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// FORCE LA DIMENSION DE LA SURFACE DE DESSIN
// 
void draw_set_size_request( void )
{
	gtk_widget_set_size_request(
			VarDraw.AdrDrawingarea,
			(gint)((gdouble)VarDraw.image_width * VarDraw.scale),
			(gint)((gdouble)VarDraw.image_height * VarDraw.scale) );
}
// 
// AFFICHAGE DU POURCENTAGE DU ZOOM
// PARAM:
//	gdouble p_zoom_scale:	0.2 .. 6.0
// 
void draw_print_zoom_changed( gdouble p_zoom_scale )
{
	gchar	*Str = NULL;
	
	Str = g_strdup_printf( "  %3.0f%%  ", 100.0 * p_zoom_scale );
	gtk_label_set_text( GTK_LABEL(GLADE_GET_OBJECT("label_zoom")), Str );
	g_free( Str );
	Str = NULL;
}
// 
// RETOURNE LE POURCENTAGE DU ZOOM EN FONCTION DE LA TAILLE DE LA FENETRE
// PARAM:
//	gdouble p_zoom_scale:	0.2 .. 6.0
// RETURN:
//	gdouble percent_zoom
// 
gdouble draw_get_scale_adjust( void )
{
	gdouble	return_scale = 1.0;
	
	if( TRUE == VarDraw.BoolScaleAdjust ) {
		gdouble	scale_width;
		gdouble	scale_height;
		// La definition SCALE_MIN correspond au pourcentage de representation le plus petit de l'image de fond
		// SCALE_MIN doit etre adaptee suivant la taille la plus petite de la fenetre
		if( VarDraw.scale > SCALE_MIN ) {
			VarDraw.scale = SCALE_MIN;
			draw_set_size_request();
			// while (gtk_events_pending()) gtk_main_iteration();
		}
		scale_width  = (gdouble)VarDraw.AdrDrawingarea->allocation.width / (gdouble)VarDraw.image_width;
		scale_height = (gdouble)VarDraw.AdrDrawingarea->allocation.height / (gdouble)VarDraw.image_height;
		
		return_scale = MIN(scale_width, scale_height );
	}
	else {
		return_scale = VarDraw.scale;
	}
	draw_print_zoom_changed( return_scale );
	return( return_scale );
}
// 
// 
// 
// 
void draw_print_top_page( cairo_t *p_cr, gdouble p_PosY, gint p_month )
{
	gchar	*Str = NULL;
	guint 	year, month, day;
	gdouble	Cpt;
	
	gtk_calendar_get_date( VarCallbacks.Adr_Calendar, &year, &month, &day );
	cairo_set_font_size( p_cr, 14 );
	cairo_set_source_rgb( p_cr, 0.0, 0.0, 0.0 );
	cairo_select_font_face(
		p_cr,
		VarDraw.FontName,
		CAIRO_FONT_SLANT_NORMAL,
		CAIRO_FONT_WEIGHT_BOLD
		);
	cairo_move_to( p_cr, 55.0, p_PosY +7 );
	Str = g_strdup_printf( "%s %d",  libutils_get_str_month( p_month ), year );
	cairo_show_text( p_cr, Str );
	g_free( Str );
	Str = NULL;
	
	// SOULIGNEMENT
	p_PosY += 15.0;
	for( Cpt = 0; Cpt < 5; Cpt ++ ) {
		cairo_move_to( p_cr, 55.0, p_PosY );
		cairo_rel_line_to( p_cr, 485.0, 0.0 );
		p_PosY += 0.5;
	}
	cairo_stroke( p_cr );
}
// 
// 
// 
// 
void draw_print_low_page( cairo_t *p_cr, gdouble p_PosY, gint p_Page )
{
	gchar			*Str = NULL;
	cairo_text_extents_t	extents;
	gdouble			PosX;
	gint			Cpt;
	
	cairo_set_font_size( p_cr, 10 );
	cairo_set_source_rgb( p_cr, 0.0, 0.0, 0.0 );
	cairo_select_font_face(
		p_cr,
		VarDraw.FontName,
		CAIRO_FONT_SLANT_ITALIC,
		CAIRO_FONT_WEIGHT_BOLD
		);
	Str = g_strdup_printf( "Page %d", p_Page );
	cairo_text_extents( p_cr, Str, &extents );
	PosX = VarDraw.image_width / 2;
	PosX -= extents.width / 2.;
	
	cairo_move_to( p_cr, PosX, p_PosY +7 );
	cairo_show_text( p_cr, Str );
	g_free( Str );
	Str = NULL;
	
	// SOULIGNEMENT
	p_PosY -= 10.0;
	for( Cpt = 0; Cpt < 5; Cpt ++ ) {
		cairo_move_to( p_cr, 55.0, p_PosY );
		cairo_rel_line_to( p_cr, 485.0, 0.0 );
		p_PosY += 0.5;
	}
	cairo_stroke( p_cr );
}
// 
// 
void draw_page( cairo_t *p_cr )
{
	guint 			year, month, day;
	GList 			*PageMonth = NULL;
	gint			*NbrPages = NULL;
	gint			*NbrLinesPerPage = NULL;
	GList			*ListPages = NULL;
	LINE_OF_PAGE		*Details = NULL;
	// cairo_text_extents_t	extents;

	gtk_calendar_get_date( VarCallbacks.Adr_Calendar, &year, &month, &day );
	page_get( month, &PageMonth, &NbrPages, &NbrLinesPerPage );

	if( NULL != PageMonth && VarEdit.YearEdit == year ) {
		
		// CLIPPING DE LA ZONE POUR NE PAS DEBORDER A DROITE
		cairo_rectangle( p_cr,
				 0,
				 0,
				 VarDraw.image_width,
				 VarDraw.image_height
				 );	
		cairo_clip( p_cr );
		cairo_new_path( p_cr );
		
		// h = ((gdouble)VarDraw.image_height / (gdouble)VarDraw.TotalPages);
		
		ListPages = g_list_first( PageMonth );
		while( ListPages ) {
			if( NULL != (Details = (LINE_OF_PAGE *)ListPages->data )) {
							
				// AFFICHAGE
				if( 	(( STRING & Details->LineType ) == STRING ) ||
					(( STRING_REPAIR & Details->LineType ) == STRING_REPAIR ) ||
					(( STRING_COMMENT & Details->LineType ) == STRING_COMMENT ) ) {
				
					cairo_set_font_size( p_cr, Details->SizeFont );
				
					if( COLOR_BLACK == Details->TypeColor )
						cairo_set_source_rgb( p_cr, 0.0, 0.0, 0.0 );
					else	cairo_set_source_rgb( p_cr, 1.0, 0.0, 0.0 );

					cairo_select_font_face(
						p_cr,
						VarDraw.FontName,
						Details->FontSlantType == FONT_SLANT_NORMAL ? CAIRO_FONT_SLANT_NORMAL : CAIRO_FONT_SLANT_ITALIC,
						Details->FontWeightType == FONT_WEIGHT_NORMAL ? CAIRO_FONT_WEIGHT_NORMAL : CAIRO_FONT_WEIGHT_BOLD
						);
				
					// CALCUL LONGUEUR DE CHAINE
					// cairo_text_extents( p_cr, Details->Str, &extents );
					// Details->LenStr = extents.width;
				
					cairo_move_to( p_cr, Details->PosX, Details->PosY );
					cairo_show_text( p_cr, Details->Str );
				}
			}
			ListPages = g_list_next( ListPages );
		}
	}
}
// 
// AFFICHAGE DE LA ZONE DE DESSIN
// PARAM:
// RETURN:
// 
void draw_paint( cairo_surface_t *p_cs )
{
	cairo_t			*cr;
	gint			canvas_w, canvas_h;
	gint			Page;
	gdouble			h;
	gdouble			PosY;
	guint 			year, month, day;
	GList 			*PageMonth = NULL;
	gint			*NbrPages = NULL;
	gint			*NbrLinesPerPage = NULL;

	// PRINT_FUNC_LF();
	
	gtk_calendar_get_date( VarCallbacks.Adr_Calendar, &year, &month, &day );

	page_get( month, &PageMonth, &NbrPages, &NbrLinesPerPage );
	if( NULL != PageMonth && VarEdit.YearEdit == year )
		Page = *NbrPages;
	else	Page = 1;
	callbacks_alloc_page( Page );
	VarDraw.scale = draw_get_scale_adjust();
	if( VarDraw.BoolScaleAdjust == FALSE ) draw_set_size_request();
	
	if( FALSE == VarDraw.BoolSaveToFile ) {
		cr = gdk_cairo_create( VarDraw.AdrDrawingarea->window );
	}
	else {
		cr = cairo_create( p_cs );
	}
	
	cairo_save( cr );
	
	canvas_w = VarDraw.AdrDrawingarea->allocation.width;
	canvas_h = VarDraw.AdrDrawingarea->allocation.height;
	VarDraw.x0 = ((canvas_w / VarDraw.scale) - VarDraw.image_width) / 2.0;
	VarDraw.y0 = ((canvas_h / VarDraw.scale) - VarDraw.image_height) / 2.0;

	cairo_scale( cr, VarDraw.scale, VarDraw.scale );
	cairo_translate( cr, VarDraw.x0, VarDraw.y0 );
	cairo_set_source_surface( cr, VarDraw.image, 0.0, 0.0 );
	cairo_paint( cr );	
	
	h = ((gdouble)VarDraw.image_height / (gdouble)VarDraw.TotalPages);
	
	// PAINT PREVIEW SURFACE: BLACK
	cairo_set_line_width( cr, 0.5 );
	for( Page = 0; Page < VarDraw.TotalPages; Page ++ ) {
		
		PosY = Page * h;
		// PAINT PREVIEW SURFACE: BLACK
		cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
		cairo_rectangle( cr,
				 3,
				 PosY +3,
				 VarDraw.image_width,
				 h -5
				 );	
		cairo_fill ( cr );

		// PAINT PREVIEW SURFACE: WHITE
		cairo_set_source_rgb( cr, 1.0, 1.0, 1.0 );
		cairo_rectangle( cr,
				 0,
				 PosY +2,
				 VarDraw.image_width,
				 h -7
				 );	
		cairo_fill ( cr );
		
		// BORDER LINE AROUND: BLACK
		cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
		cairo_rectangle( cr,
				 0,
				 PosY +2,
				 VarDraw.image_width,
				 h -7
				 );
		
		// 
		// CONTOURS: HAUT DE PAGE / MILIEU  DE PAGE / BAS DE PAGE
		// 
		/*cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
		cairo_rectangle( cr,
				 5,
				 PosY +5,
				 VarDraw.image_width -10,
				 45
				 );
		cairo_rectangle( cr,
				 5,
				 PosY +55,
				 VarDraw.image_width -10,
				 735
				 );
		cairo_rectangle( cr,
				 5,
				 PosY +795,
				 VarDraw.image_width -10,
				 35
				 );*/
		// HAUT DE PAGE
		draw_print_top_page( cr, PosY + 25, month );
		// BAS DE PAGE
		draw_print_low_page( cr, PosY + h - 25 - 7, Page +1 );
		
		cairo_stroke( cr );
	}
	
	// AFFICHAGE
	cairo_select_font_face( cr, VarDraw.FontName, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL );
	cairo_set_font_size( cr, (gdouble)VarDraw.SizeFontName );
	cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
	
	draw_page( cr  );
	
	cairo_restore (cr);
	cairo_destroy( cr );
}

